from pathlib import Path
from typing import Annotated

import yaml

from file_operations.local_files import LocalFile
from cvs.github import GithubConnector
from cvs.github_settings import GithubSettings
from models import Software, GitlabSoftware, GithubSoftware

import typer

from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.console import Console
from rich.theme import Theme

log_theme = Theme(
    {
        "info": "cyan",
        "warning": "yellow",
        "error": "bold red",
    }
)
console = Console(theme=log_theme)

github_settings = GithubSettings()
github = GithubConnector(github_settings)


def load_software_list(software_file: Path) -> list[Software]:
    software_yaml = yaml.safe_load(software_file.open())
    software_list = []
    for software in software_yaml.get("software", []):
        match software.get("cvs").lower():
            case "github":
                software_list.append(GithubSoftware(**software))
            case "gitlab":
                software_list.append(GitlabSoftware(**software))
            case _:
                pass
    return software_list


def is_software_to_update(software: Software) -> bool:
    return LocalFile(software).is_file_absent()


def update(software: Software) -> None:
    if isinstance(software, GithubSoftware):
        download = github.download_software(software)
        local_file = LocalFile(software)
        local_file.write_file(download)
        local_file.chmod(0o544)
        local_file.chmod_other_versions(0o444)


def main(
    file_path: Annotated[
        str,
        typer.Option(
            "--file",
            "-f",
            help="Configuration file that contains the definition of the software to get and their version",
        ),
    ] = "./software_definition.yml"
):
    path = Path(file_path).resolve()
    software_list = load_software_list(path)
    for software in software_list:
        if is_software_to_update(software):
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.desciption]{task.description}"),
                transient=True,
            ) as progress:
                progress.add_task(description=f"Updating {software.name}...")
                update(software)
        else:
            console.print(
                f"[bold]{software.name}[/bold] is up to date.",
                style="info",
            )


if __name__ == "__main__":
    typer.run(main)
